/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: dd_complex.h,v 1.3 2009/09/24 05:39:01 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

/*
Complex class declare for the GMP.
*/

#ifndef _DD_COMPLEX_H_
#define _DD_COMPLEX_H_

#include <complex>

class dd_complex {
  public:
    dd_real re, im;
    dd_complex() {
	re = 0.0;
	im = 0.0;
    } dd_complex(dd_real i, dd_real j) {
	re = i;
	im = j;
    }
    dd_complex(dd_real i) {
	re = i;
	im = 0.0;
    }

    bool operator==(dd_complex & other);
    bool operator!=(dd_complex & other);

    bool operator==(dd_real & other);
    bool operator!=(dd_real & other);

    void operator=(std::complex < double >ob1);
    void operator=(double ob1);
    void operator=(dd_real ob1);

    friend dd_complex operator+(dd_complex ob1, dd_complex ob2);
    friend dd_complex operator+(dd_complex ob1, dd_real ob2);
    friend dd_complex operator+(dd_real ob1, dd_complex ob2);
    friend dd_complex operator+(dd_complex ob1, std::complex < double >ob2);

    friend dd_complex operator-(dd_complex ob1, dd_complex ob2);
    friend dd_complex operator-(dd_complex ob1);
    friend dd_complex operator-(dd_complex ob1, dd_real ob2);
    friend dd_complex operator-(dd_real ob1, dd_complex ob2);
    friend dd_complex operator-(dd_real ob1, std::complex < double >ob2);

    friend dd_complex operator*(dd_complex ob1, dd_complex ob2);
    friend dd_complex operator*(dd_complex ob1, dd_real ob2);
    friend dd_complex operator*(dd_real ob1, dd_complex ob2);

    friend dd_complex operator/(dd_complex ob1, dd_complex ob2);
    friend dd_complex operator/(dd_complex ob1, dd_real ob2);
    friend dd_complex operator/(dd_real ob1, dd_complex ob2);

};



inline bool
dd_complex::operator==(dd_complex & other)
{
    if ((this->re == other.re) && (this->im == other.im)) {
	return true;
    } else
	return false;
}

inline bool
dd_complex::operator==(dd_real & other)
{
    if ((this->re == other) && (this->im == 0.0)) {
	return true;
    } else
	return false;
}


inline bool
dd_complex::operator!=(dd_complex & other)
{
    if ((this->re != other.re) || (this->im != other.im)) {
	return true;
    } else
	return false;
}

inline bool
dd_complex::operator!=(dd_real & other)
{
    if ((this->re != other) || (this->im != 0.0)) {
	return true;
    } else
	return false;
}

inline void
    dd_complex::operator=(dd_real ob1)
{
    re = ob1;
    im = 0.0;
}

inline void
    dd_complex::operator=(double ob1)
{
    re = ob1;
    im = 0.0;
}

inline void
   dd_complex::operator=(std::complex < double >ob1)
{
    re = ob1.real();
    im = ob1.imag();
}

inline dd_complex
operator+(dd_complex ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re + ob2.re;
    tmp.im = ob1.im + ob2.im;
    return tmp;
}

inline dd_complex
operator+(dd_real ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1 + ob2.re;
    tmp.im = ob2.im;
    return tmp;
}

inline dd_complex
operator+(dd_complex ob1, dd_real ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re + ob2;
    tmp.im = ob1.im;
    return tmp;
}

inline dd_complex
operator+(dd_complex ob1, std::complex < double >ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re + ob2.real();
    tmp.im = ob1.im + ob2.imag();
    return tmp;
}

inline dd_complex
operator-(dd_complex ob1, std::complex < double >ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re - ob2.real();
    tmp.im = ob1.im - ob2.imag();
    return tmp;
}

inline dd_complex
operator-(dd_complex ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re - ob2.re;
    tmp.im = ob1.im - ob2.im;
    return tmp;
}

inline dd_complex
operator-(dd_complex ob1)
{
    dd_complex tmp;

    tmp.re = -ob1.re;
    tmp.im = -ob1.im;
    return tmp;
}


inline dd_complex
operator-(dd_real ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1 - ob2.re;
    tmp.im = -ob2.im;
    return tmp;
}

inline dd_complex
operator-(dd_complex ob1, dd_real ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re - ob2;
    tmp.im = ob1.im;
    return tmp;
}

inline dd_complex
operator*(dd_complex ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re * ob2.re - ob1.im * ob2.im;
    tmp.im = ob1.re * ob2.im + ob1.im * ob2.re;
    return tmp;
}

inline dd_complex
operator*(dd_real ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = ob1 * ob2.re;
    tmp.im = ob1 * ob2.im;
    return tmp;
}

inline dd_complex
operator*(dd_complex ob1, dd_real ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re * ob2;
    tmp.im = ob1.im * ob2;
    return tmp;
}

inline dd_complex
operator/(dd_complex ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re =
	(ob1.re * ob2.re + ob1.im * ob2.im) / (ob2.re * ob2.re +
	ob2.im * ob2.im);
    tmp.im =
	(ob1.im * ob2.re - ob1.re * ob2.im) / (ob2.re * ob2.re +
	ob2.im * ob2.im);
    return tmp;
}

inline dd_complex
operator/(dd_real ob1, dd_complex ob2)
{
    dd_complex tmp;

    tmp.re = (ob1 * ob2.re) / (ob2.re * ob2.re + ob2.im * ob2.im);
    tmp.im = -(ob1 * ob2.im) / (ob2.re * ob2.re + ob2.im * ob2.im);
    return tmp;
}

inline dd_complex
operator/(dd_complex ob1, dd_real ob2)
{
    dd_complex tmp;

    tmp.re = ob1.re / ob2;
    tmp.im = ob1.im / ob2;
    return tmp;
}

inline dd_real
abs(dd_complex ctmp)
{
    return sqrt(ctmp.re * ctmp.re + ctmp.im * ctmp.im);
}

inline dd_complex
sqrt(dd_complex z)
{
    dd_real
	mag;
    dd_complex
	r;

    mag = abs(z);

    if (abs(mag) == 0.0) {
	r.re = 0.0, r.im = 0.0;
    } else if (z.re > 0.0) {
	r.re = sqrt(0.5 * (mag + z.re));
	r.im = z.im / (2.0 * r.re);
    } else {
	r.im = sqrt(0.5 * (mag - z.re));
	if (z.im < 0.0)
	    r.im = -r.im;
	r.re = z.im / (2.0 * r.im);
    }
    return r;
}

inline dd_complex
conjg(dd_complex ctmp)
{
    dd_complex ctmp2;

    ctmp2.re = ctmp.re;
    ctmp2.im = -ctmp.im;
    return ctmp2;
}

#endif
