/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cgeru.debug.cpp,v 1.8 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_INCX  -2
#define MAX_INCX   8
#define MIN_INCY  -2
#define MAX_INCY   8
#define MIN_N     -2
#define MAX_N      8
#define MIN_M     -2
#define MAX_M      8
#define MIN_LDA   -2
#define MAX_LDA    8
#define MAX_ITER   5

void
Cgeru_test()
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = MIN_N; n < MAX_N; n++) {
		for (int m = MIN_M; m < MAX_M; m++) {
		    for (int lda = m; lda < MAX_LDA; lda++) {
#if defined VERBOSE_TEST
			printf
			    ("#m is %d, n is %d, lda is %d, incx is %d, incy is %d\n",
			    m, n, lda, incx, incy);
#endif
			COMPLEX *A;
			COMPLEX *x;
			COMPLEX *y;
			complex < double >*Ad;
			complex < double >*xd;
			complex < double >*yd;

			A = new COMPLEX[matlen(lda, n)];
			x = new COMPLEX[veclen(m, incx)];
			y = new COMPLEX[veclen(n, incy)];
			Ad = new complex < double >[matlen(lda, n)];
			xd = new complex < double >[veclen(m, incx)];
			yd = new complex < double >[veclen(n, incy)];

			COMPLEX alpha;
			complex < double >alphad;

			for (int i = 0; i < MAX_ITER; i++) {
			    set_random_number(&alpha, &alphad);

			    set_random_vector(A, Ad, matlen(lda, n));
			    set_random_vector(x, xd, veclen(m, incx));
			    set_random_vector(y, yd, veclen(n, incy));

		            mpack_errno = 0; blas_errno = 0;
			    Cgeru(m, n, alpha, x, incx, y, incy, A, lda);
			    zgeru_f77(&m, &n, &alphad, xd, &incx, yd, &incy,
				Ad, &lda);

#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }

			    double diff = infnorm(A, Ad, matlen(lda, n), 1);
			    if (fabs(diff) > EPSILON) {
				printf("#error %.16e!!\n", diff);
				errorflag = TRUE;
			    }
			}
			delete[]yd;
			delete[]xd;
			delete[]Ad;
			delete[]y;
			delete[]x;
			delete[]A;
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cgeru test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cgeru_test();
    printf("Cgeru test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
