/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Chpr.debug.cpp,v 1.7 2009/09/15 12:49:54 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_INCX   -2
#define MAX_INCX    3
#define MIN_N       2
#define MAX_N      40
#define MAX_ITER   10

void
Chpr_test2(const char *uplo)
{
    int errorflag = FALSE;
    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n <= MAX_N; n++) {
#if defined VERBOSE_TEST
	  printf("#n is %d, incx is %d ", n, incx);
	  printf("uplo is %s \n", uplo);
#endif
	    REAL alpha;
	    COMPLEX *x;
	    COMPLEX *AP;
	    double alphad;
	    complex < double >*xd;
	    complex < double >*APd;

	    x = new COMPLEX[veclen(n, incx)];
	    AP = new COMPLEX[veclen(n * (n + 1) / 2, 1)];
	    xd = new complex < double >[veclen(n, incx)];
	    APd = new complex < double >[veclen(n * (n + 1) / 2, 1)];

	    for (int i = 0; i < MAX_ITER; i++) {
		set_random_vector(AP, APd, veclen(n * (n + 1) / 2, 1));
		set_random_vector(x, xd, veclen(n, incx));
		set_random_number(&alpha, &alphad);

	        mpack_errno = 0; blas_errno = 0;
		Chpr(uplo, n, alpha, x, incx, AP);
		zhpr_f77(uplo, &n, &alphad, xd, &incx, APd);
#if defined VERBOSE_TEST
	    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif	
	      if (mpack_errno != blas_errno) {
		printf("error in Mxerbla!!\n");
		exit(1);
	      }
	      
		double diff = infnorm(AP, APd, veclen(n * (n + 1) / 2, 1), 1);
		if (diff > EPSILON) {
		    printf("#error %.16e!!\n", diff);
		    printf("#n is %d, incx is %d, uplo is %s.\n", n, incx,
			uplo);
		    errorflag = TRUE;
		}
	    }

	    delete[]APd;
	    delete[]xd;
	    delete[]AP;
	    delete[]x;
	}
    }
    if (errorflag) {
	printf("Chpr test has failed...\n");
	exit(1);
    }
}

void
Chpr_test()
{
    Chpr_test2("U");
    Chpr_test2("L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Chpr_test();
    printf("Chpr test has passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
