/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: RCabs1.debug.cpp,v 1.3 2009/09/15 11:55:45 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_ITER 10

void
RCabs1_test()
{
    int errorflag = FALSE;
    COMPLEX z;
    complex < double >dz;
    REAL Ftemp;
    double dtemp;
    int j = 0;

    while (j < MAX_ITER) {
	z = mpc_randomnumber();
	dz.real() = cast2double(z.re);
	dz.imag() = cast2double(z.im);

	Ftemp = RCabs1(z);
	dtemp = dcabs1_f77(&dz);
#if defined VERBOSE_TEST
	    printf("Z   ="); printnum (z); printf("\n");
	    printf("dz  ="); printnum (dz); printf("\n");

	    printf("RCabs1 = "); printnum (Ftemp); printf("\n");
	    printf("dcabs1 = "); printnum (dtemp); printf("\n");
#endif
	double diff = cast2double(Ftemp) - dtemp;
	if (fabs(diff) > EPSILON) {
	    printf("error!!\n");
	    errorflag = TRUE;
	}
	j++;
    }
    if (errorflag == TRUE) {
	printf("RCabs1 test failed...\n");
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    RCabs1_test();
    printf("RCabs1 test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
