/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rgemm.debug.cpp,v 1.5 2009/09/15 12:49:54 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_M -2
#define MAX_M 10
#define MIN_N -2
#define MAX_N 10
#define MIN_K -2
#define MAX_K 10
#define MIN_LDA -2
#define MAX_LDA 10
#define MIN_LDB -2
#define MAX_LDB 10
#define MIN_LDC -2
#define MAX_LDC 10
#define MAX_ITER 3

void
Rgemm_test3(const char *transa, const char *transb, REAL alpha,
    REAL beta)
{
    int errorflag = FALSE;

    for (int k = MIN_K; k < MAX_K; k++) {
	for (int n = MIN_N; n < MAX_N; n++) {
	    for (int m = MIN_M; m < MAX_M; m++) {

		int minlda, minldb;

		if (Mlsame(transa, "N"))
		    minlda = max(1, m);
		else
		    minlda = max(1, k);

		if (Mlsame(transb, "N"))
		    minldb = max(1, k);
		else
		    minldb = max(1, n);

		for (int lda = minlda; lda < MAX_LDA; lda++) {
		    for (int ldb = minldb; ldb < MAX_LDB; ldb++) {
			for (int ldc = max(1, m); ldc < MAX_LDC; ldc++) {
#if defined VERBOSE_TEST
			    printf
				("#k is %d, n is %d, m is %d, lda is %d, ldb is %d, ldc is %d\n",
				k, n, m, lda, ldb, ldc);
			    printf("#transa is %s, transb is %s\n", transa,
				transb);
#endif
			    REAL *A;
			    REAL *B;
			    REAL *C;
			    double *Ad;
			    double *Bd;
			    double *Cd;

			    int ka, kb;

			    if (Mlsame(transa, "N"))
				ka = k;
			    else
				ka = m;

			    if (Mlsame(transb, "N"))
				kb = n;
			    else
				kb = k;

			    A = new REAL[matlen(lda, ka)];
			    B = new REAL[matlen(ldb, kb)];
			    C = new REAL[matlen(ldc, n)];
			    Ad = new double[matlen(lda, ka)];
			    Bd = new double[matlen(ldb, kb)];
			    Cd = new double[matlen(ldc, n)];

			    double alphad, betad;

			    alphad = cast2double(alpha);
			    betad = cast2double(beta);

			    for (int iter = 0; iter < MAX_ITER; iter++) {
				set_random_vector(A, Ad, matlen(lda, ka));
				set_random_vector(B, Bd, matlen(ldb, kb));
				set_random_vector(C, Cd, matlen(ldc, n));

			        mpack_errno = 0; blas_errno = 0;
				Rgemm(transa, transb, m, n, k, alpha, A, lda,
				    B, ldb, beta, C, ldc);
				dgemm_f77(transa, transb, &m, &n, &k, &alphad,
				    Ad, &lda, Bd, &ldb, &betad, Cd, &ldc);
                               #if defined VERBOSE_TEST
				printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
                                	    #endif	
				if (mpack_errno != blas_errno) {
				  printf("error in Mxerbla!!\n");
				  exit(1);
				}
			      double diff;
			      diff = infnorm(C, Cd, matlen(ldc, n), 1);
			      if (fabs(diff) > EPSILON) {
				    printf("#error %.16e!!\n", diff);
				    errorflag = TRUE;
			      }
			    }
			    delete[]Cd;
			    delete[]Bd;
			    delete[]Ad;

			    delete[]C;
			    delete[]B;
			    delete[]A;
			}
		    }
		}
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Rgemm test failed...\n");
	exit(1);
    }
}

void
Rgemm_test2(const char *uplo, const char *trans)
{
    REAL alpha;
    REAL beta;
    REAL Rzero, Rone;

    Rzero = 0.0;
    Rone = 1.0;

//a=0, b=*;
    alpha = Rzero;
    beta = mpf_randomnumber();
    Rgemm_test3(uplo, trans, alpha, beta);

//a=*, b=0;
    alpha = mpf_randomnumber();
    beta = Rzero;
    Rgemm_test3(uplo, trans, alpha, beta);

//a=*, b=1;
    alpha = mpf_randomnumber();
    beta = Rone;
    Rgemm_test3(uplo, trans, alpha, beta);

//alpha=*, beta=*
    alpha = mpf_randomnumber();
    beta = mpf_randomnumber();
    Rgemm_test3(uplo, trans, alpha, beta);

//a=0, b=0;
    alpha = Rzero;
    beta = Rzero;
    Rgemm_test3(uplo, trans, alpha, beta);

//a=0, b=1;
    alpha = Rzero;
    beta = Rone;
    Rgemm_test3(uplo, trans, alpha, beta);

//a=1, b=0;
    alpha = Rone;
    beta = Rzero;
    Rgemm_test3(uplo, trans, alpha, beta);

//a=1, b=1;
    alpha = Rone;
    beta = Rone;
    Rgemm_test3(uplo, trans, alpha, beta);

//a=1, b=*;
    alpha = Rone;
    beta = mpf_randomnumber();
    Rgemm_test3(uplo, trans, alpha, beta);
}

void
Rgemm_test()
{
    Rgemm_test2("N", "N");
    Rgemm_test2("N", "T");
    Rgemm_test2("N", "C");

    Rgemm_test2("T", "N");
    Rgemm_test2("T", "C");
    Rgemm_test2("T", "T");

    Rgemm_test2("C", "N");
    Rgemm_test2("C", "T");
    Rgemm_test2("C", "C");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rgemm_test();
    printf("Rgemm test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
