/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rrot.debug.cpp,v 1.5 2009/09/16 00:55:18 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  10

void
Rrot_test()
{
    int errorflag = FALSE;
    REAL ctemp;
    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 3; n < MAX_N; n++) {
#if defined VERBOSE_TEST
                printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);
#endif
		REAL *x = new REAL[veclen(n, incx)];
		REAL *y = new REAL[veclen(n, incy)];
		double *xd = new double[veclen(n, incx)];
		double *yd = new double[veclen(n, incy)];

		REAL c, s;
		double cd, sd;

		set_random_number(&c, &cd);
		set_random_number(&s, &sd);

		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(x, xd, veclen(n, incx));
		    set_random_vector(y, yd, veclen(n, incy));

		    Rrot(n, x, incx, y, incy, c, s);
		    drot_f77(&n, xd, &incx, yd, &incy, &cd, &sd);

		    double diff = infnorm(x, xd, n, abs(incx));
		    if (fabs(diff) > EPSILON) {
			printf("error %.16e!!\n", diff);
			errorflag = TRUE;
		    }
		    diff = infnorm(y, yd, n, abs(incy));
		    if (fabs(diff) > EPSILON) {
			printf("error %.16e!!\n", diff);
			errorflag = TRUE;
		    }

		    j++;
		}
		delete[]x;
		delete[]y;
		delete[]xd;
		delete[]yd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rrot test failed...\n");
	exit(1);
    }
}


int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rrot_test();
    printf("Rrot test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

