/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rswap.debug.cpp,v 1.6 2009/09/25 04:00:39 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_INCX  -5
#define MAX_INCX  10
#define MIN_INCY  -5
#define MAX_INCY  10
#define MAX_N     100
#define MAX_ITER  10

void
Rswap_test()
{
    int errorflag = FALSE;
    REAL rtemp;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = 0; n < MAX_N; n++) {
		printf("# n:%d incx:%d incy:%d\n", n, incx, incy);
		REAL *x = new REAL[veclen(n, incx)];
		double *xd = new double[veclen(n, incx)];

		REAL *y = new REAL[veclen(n, incy)];
		double *yd = new double[veclen(n, incy)];

		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(x, xd, veclen(n, incx));
		    set_random_vector(y, yd, veclen(n, incy));

		    Rswap(n, x, incx, y, incy);
		    dswap_f77(&n, xd, &incx, yd, &incy);

		    double diff;

		    diff = infnorm(x, xd, veclen(n, incx), 1);
		    diff = diff + infnorm(y, yd, veclen(n, incy), 1);

		    if (fabs(diff) > EPSILON) {
			printf("error %.16e!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]xd;
		delete[]x;
		delete[]yd;
		delete[]y;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rswap test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rswap_test();
    printf("Rswap test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}


