/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rtpsv.debug.cpp,v 1.6 2009/09/16 03:06:20 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N -1
#define MAX_N 10
#define MIN_INCX -3
#define MAX_INCX 3
#define MAX_ITER 5

void
Rtpsv_test2(const char *uplo, const char *trans, const char *diag)
{
    int errorflag = FALSE;
    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n < MAX_N; n++) {
#if defined VERBOSE_TEST
	    printf("#n is %d, incx is %d ", n, incx);
	    printf("uplo is %s trans is %s, diag is %s \n", uplo, trans, diag);
#endif
	    REAL *AP = new REAL[veclen(n * (n + 1) / 2, 1)];
	    REAL *x = new REAL[veclen(n, incx)];
	    double *APd = new double[veclen(n * (n + 1) / 2, 1)];
	    double *xd = new double[veclen(n, incx)];

	    for (int iter = 0; iter < MAX_ITER; iter++) {
		set_random_vector(AP, APd, veclen(n * (n + 1) / 2, 1));
		set_random_vector(x, xd, veclen(n, incx));

                mpack_errno = 0; blas_errno = 0;
		Rtpsv(uplo, trans, diag, n, AP, x, incx);
		dtpsv_f77(uplo, trans, diag, &n, APd, xd, &incx);
#if defined VERBOSE_TEST
	        printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
	        if (mpack_errno != blas_errno) {
		    printf("error in Mxerbla!!\n");
		    exit(1);
		}
		double diff = infnorm(x, xd, veclen(n, incx), 1);
		if (fabs(diff) > EPSILON) {
		    printf("#error %.16e!!\n", diff);
		    printf("#n is %d, incx is %d ", n, incx);
		    printf("uplo is %s trans is %s, diag is %s \n", uplo,
			trans, diag);
		    errorflag = TRUE;
		}
	    }
	    delete[]APd;
	    delete[]AP;
	    delete[]xd;
	    delete[]x;
	}
    }
    if (errorflag == TRUE) {
	printf("Rtpsv test failed...\n");
	exit(1);
    }
}

void
Rtpsv_test()
{
    Rtpsv_test2("U", "N", "U");
    Rtpsv_test2("U", "N", "N");
    Rtpsv_test2("U", "T", "U");
    Rtpsv_test2("U", "T", "N");
    Rtpsv_test2("U", "C", "U");
    Rtpsv_test2("U", "C", "N");

    Rtpsv_test2("L", "N", "U");
    Rtpsv_test2("L", "N", "N");
    Rtpsv_test2("L", "T", "U");
    Rtpsv_test2("L", "T", "N");
    Rtpsv_test2("L", "C", "U");
    Rtpsv_test2("L", "C", "N");

}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rtpsv_test();
    printf("Rtpsv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}


