/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clargv.cpp,v 1.3 2009/09/16 06:00:47 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include <mblas.h>
#include <mlapack.h>


inline REAL
abs1(COMPLEX ff)
{
    return max(abs(ff.re), abs(ff.im));
}

inline REAL
abssq(COMPLEX ff)
{
    REAL temp;

    temp = (ff.re * ff.re) + (ff.im * ff.im);
    return temp;
}

void
Clargv(INTEGER n, COMPLEX * x, INTEGER incx, COMPLEX * y, INTEGER incy, REAL * c,
    INTEGER incc)
{
    REAL d;
    COMPLEX f, g;
    INTEGER i, j;
    COMPLEX r;
    REAL f2, g2;
    INTEGER ic;
    REAL di;
    COMPLEX ff;
    REAL cs, dr;
    COMPLEX fs, gs;
    INTEGER ix, iy;
    COMPLEX sn;
    REAL f2s, g2s, eps, scale;
    INTEGER count, itemp;
    REAL safmn2;
    REAL safmx2;
    REAL safmin;
    REAL Zero = 0.0, One = 1.0;

    safmin = Rlamch("S");
    eps = Rlamch("E");

    safmn2 = sqrt(safmin / eps);
    itemp = (INTEGER)cast2double(safmn2);
    safmn2 = 2 ^ itemp;
    safmx2 = One / safmn2;

    ix = 1;
    iy = 1;
    ic = 1;
    for (i = 0; i < n; i++) {
	f = x[ix];
	g = y[iy];
//Use identical algorithm as in ZLARTG
	scale = max(abs1(f), abs1(g));
	fs = f;
	gs = g;
	count = 0;
	if (scale >= safmx2) {
	  L10:
	    ++count;
	    fs = fs * safmn2;
	    gs = gs * safmn2;
	    scale = scale * safmn2;
	    if (scale >= safmx2) {
		goto L10;
	    }
	} else if (scale <= safmn2) {
	    if (g == Zero) {
		cs = One;
		sn = Zero;
		r = f;
		goto L50;
	    }
	  L20:
	    --count;
	    fs = fs * safmx2;
	    gs = gs * safmx2;
	    scale = scale * safmx2;
	    if (scale <= safmn2) {
		goto L20;
	    }
	}
	f2 = abssq(fs);
	g2 = abssq(gs);
	if (f2 <= max(g2, One) * safmin) {
//This is a rare case: F is very small.
	    if (f == Zero) {
		cs = Zero;
		r = Rlapy2(g.re, g.im);
//Do complex/real division explicitly with two real
//divisions
		d = Rlapy2(gs.re, gs.im);
		sn = Real2Complex(gs.re / d, -gs.im / d);
		goto L50;
	    }
	    f2s = Rlapy2(fs.re, fs.im);
//G2 and G2S are accurate
//G2 is at least SAFMIN, and G2S is at least SAFMN2
	    g2s = sqrt(g2);
//Error in CS from underflow in F2S is at most
//UNFL / SAFMN2 .lt. sqrt(UNFL*EPS) .lt. EPS
//If MAX(G2,ONE)=G2, then F2 .lt. G2*SAFMIN,
//and so CS .lt. sqrt(SAFMIN)
//If MAX(G2,ONE)=ONE, then F2 .lt. SAFMIN
//and so CS .lt. sqrt(SAFMIN)/SAFMN2 = sqrt(EPS)
//Therefore, CS = F2S/G2S / sqrt( 1 + (F2S/G2S)**2 ) = F2S/G2S
	    cs = f2s / g2s;
//Make sure abs(FF) = 1
//Do complex/real division explicitly with 2 real divisions
	    if (abs1(f) > One) {
		d = Rlapy2(f.re, f.im);
		ff = Real2Complex(f.re / d, f.im / d);
	    } else {
		dr = safmx2 * f.re;
		di = safmx2 * f.im;
		d = Rlapy2(dr, di);
		ff = Real2Complex(dr / d, di / d);
	    }
	    sn = ff * Real2Complex(gs.re / g2s, -gs.im / g2s);
	    r = cs * f + sn * g;
	} else {
//This is the most common case.
//Neither F2 nor F2/G2 are less than SAFMIN
//F2S cannot overflow, and it is accurate
	    f2s = sqrt(g2 / f2 + One);
//Do the F2S(real)*FS(complex) multiply with two real
//multiplies
	    r = Real2Complex(f2s * fs.re, f2s * fs.im);
	    cs = One / f2s;
	    d = f2 + g2;
//Do complex/real division explicitly with two real divisions
	    sn = Real2Complex(r.re / d, r.im / d);
	    sn = sn * conjg(gs);
	    if (count != 0) {
		if (count > 0) {
		    for (j = 0; j < count; j++) {
			r = r * safmx2;
		    }
		} else {
		    for (j = 0; j < -count; j++) {
			r = r * safmn2;
		    }
		}
	    }
	}
      L50:
	c[ic] = cs;
	y[iy] = sn;
	x[ix] = r;
	ic += incc;
	iy += incy;
	ix += incx;
    }
    return;
}
