/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2009 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include <mblas.h>
#include <mlapack.h>

#define MTRUE 1
#define MFALSE 0

void Rlarrr(INTEGER n, REAL * d, REAL * e, INTEGER *info)
{
    INTEGER i;
    REAL eps, tmp, tmp2, rmin;
    REAL offdig, safmin;
    INTEGER yesrel;
    REAL smlnum, offdig2;
    REAL Zero = 0.0;

    *info = 1;
    safmin = Rlamch("Safe minimum");
    eps = Rlamch("Precision");
    smlnum = safmin / eps;
    rmin = sqrt(smlnum);
//Tests for relative accuracy 
//Test for scaled diagonal dominance 
//Scale the diagonal entries to one and check whether the sum of the 
//off-diagonals is less than one 
//The sdd relative error bounds have a 1/(1- 2*x) factor in them, 
//x = max(OFFDIG + OFFDIG2), so when x is close to 1/2, no relative 
//accuracy is promised.  In the notation of the code fragment below, 
//1/(1 - (OFFDIG + OFFDIG2)) is the condition number. 
//We don't think it is worth going into "sdd mode" unless the relative 
//condition number is reasonable, not 1/macheps. 
//The threshold should be compatible with other thresholds used in the 
//code. We set  OFFDIG + OFFDIG2 <= .999 =: RELCOND, it corresponds 
//to losing at most 3 decimal digits: 1 / (1 - (OFFDIG + OFFDIG2)) <= 1000 
//instead of the current OFFDIG + OFFDIG2 < 1 
    yesrel = MTRUE;
    offdig = Zero;
    tmp = sqrt(abs(d[1]));
    if (tmp < rmin) {
	yesrel = MFALSE;
    }
    if (!yesrel) {
	goto L11;
    }
    for (i = 1; i < n; i++) {
	tmp2 = sqrt(abs(d[i]));
	if (tmp2 < rmin) {
	    yesrel = MFALSE;
	}
	if (!yesrel) {
	    goto L11;
	}
	offdig2 = abs(e[i - 1]) / (tmp * tmp2);
	if (offdig + offdig2 >= .999) {
	    yesrel = MFALSE;
	}
	if (!yesrel) {
	    goto L11;
	}
	tmp = tmp2;
	offdig = offdig2;

    }
  L11:
    if (yesrel) {
	*info = 0;
	return;
    } else {
    }
//*** MORE TO BE IMPLEMENTED *** 
//Test if the lower bidiagonal matrix L from T = L D L^T 
//(zero shift facto) is well conditioned 
//Test if the upper bidiagonal matrix U from T = U D U^T 
//(zero shift facto) is well conditioned. 
//In this case, the matrix needs to be flipped and, at the end 
//of the eigenvector computation, the flip needs to be applied 
//to the computed eigenvectors (and the support) 
    return;
}
