/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cgesv.debug.cpp,v 1.1 2009/12/25 07:47:19 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N      1
#define MAX_N     10
#define MIN_NRHS   1
#define MAX_NRHS  10
#define MAX_LDA   9
#define MAX_LDB   9
#define MAX_ITER  10

void
Cgesv_test()
{
    int errorflag = FALSE;
    int j = 0;
    int infod;
    mpackint info;
    double diff;

    for (int n = MIN_N; n <= MAX_N; n++) {
	for (int nrhs = MIN_NRHS; nrhs <= n; nrhs++) {
	    for (int lda = max(n, 1); lda <= MAX_LDA; lda++) {
		for (int ldb = max(n, 1); ldb <= MAX_LDB; ldb++) {

		    COMPLEX *A = new COMPLEX[matlen(lda, n)];
		    COMPLEX *B = new COMPLEX[matlen(ldb, nrhs)];
		    mpackint *ipiv = new mpackint[veclen(n, 1)];

		    complex <double> *Ad = new complex <double>[matlen(lda, n)];
		    complex <double> *Bd = new complex <double>[matlen(ldb, nrhs)];
		    int *ipivd = new int[veclen(n, 1)];
#if defined VERBOSE_TEST
		    printf("#n:%d lda %d nrhs %d ldb %d\n", n, lda, nrhs, ldb);
#endif
		    j = 0;
		    while (j < MAX_ITER) {
			set_random_vector(A, Ad, matlen(lda, n));
			set_random_vector(B, Bd, matlen(ldb, nrhs));
			Cgesv(n, nrhs, A, lda, ipiv, B, ldb, &info);
			zgesv_f77(&n, &nrhs, Ad, &lda, ipivd, Bd, &ldb,
			    &infod);

			if (info < 0) {
			    printf("info %d error\n", -(int)info);
			}
			if (info != infod) {
			    printf("info differ! %d, %d\n", (int)info, infod);
			    errorflag = TRUE;
			}
			diff = infnorm(B, Bd, matlen(ldb, nrhs), 1);
			if (fabs(diff) > EPSILON*100.0) {
		            printf("#n:%d lda %d nrhs %d ldb %d diff %.16e\n", n, lda, nrhs, ldb, diff);
			    errorflag = TRUE;
			}
			j++;
		    }
		    delete[]ipivd;
		    delete[]Bd;
		    delete[]Ad;
		    delete[]ipiv;
		    delete[]B;
		    delete[]A;
		}
		if (errorflag == TRUE) {
		    printf("Cgesv test failed...\n");
		    exit(1);
		}
	    }
	}
    }
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cgesv_test();
    printf("Cgesv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
