/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clacrm.debug.cpp,v 1.3 2009/09/23 03:43:12 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N 0
#define MAX_N 10
#define MIN_M 0
#define MAX_M 10
#define MAX_LDA 10
#define MAX_LDB 10
#define MAX_LDC 10
#define MAX_ITER 10

void
Clacrm_test()
{
    int errorflag = FALSE;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int m = MIN_M; m < MAX_M; m++) {
	    for (int lda = max(1, m); lda < MAX_LDA; lda++) {
		for (int ldb = max(1, n); ldb < MAX_LDB; ldb++) {
		    for (int ldc = max(1, m); ldc < MAX_LDC; ldc++) {
#if defined VERBOSE_TEST
			printf("# n %d m %d lda %d ldb %d ldc %d\n", n, m, lda,
			       ldb, ldc);
#endif			    

			COMPLEX *A = new COMPLEX[matlen(lda, n)];
			REAL *B = new REAL[matlen(ldb, n)];
			COMPLEX *C = new COMPLEX[matlen(ldc, n)];
			REAL *rwork = new REAL[veclen(2 * m * n, 1)];
			complex < double >*Ad =
			    new complex < double >[matlen(lda, n)];
			double *Bd = new double[matlen(ldb, n)];
			complex < double >*Cd =
			    new complex < double >[matlen(ldc, n)];
			double *rworkd = new double[veclen(2 * m * n, 1)];

			for (int iter = 0; iter < MAX_ITER; iter++) {
			    set_random_vector(A, Ad, matlen(lda, n));
			    set_random_vector(B, Bd, matlen(ldb, n));
			    set_random_vector(C, Cd, matlen(ldc, n));

			    zlacrm_f77(&m, &n, Ad, &lda, Bd, &ldb, Cd, &ldc,
				rworkd);
			    Clacrm(m, n, A, lda, B, ldb, C, ldc, rwork);

			    double diff;

			    diff = infnorm(C, Cd, matlen(ldc, n), 1);
			    if (fabs(diff) > EPSILON) {
				printf("#error %.16e!\n", diff);
				errorflag = TRUE;
			    } else {
			    }
			}
			delete[]rworkd;
			delete[]Cd;
			delete[]Bd;
			delete[]Ad;
			delete[]rwork;
			delete[]C;
			delete[]B;
			delete[]A;
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Clacrm test failed...\n");
	exit(1);
    }
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clacrm_test();
    printf("Clacrm test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
