/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clacrt.debug.cpp,v 1.3 2009/12/28 02:13:37 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  1

void
Clacrt_test()
{
    int errorflag = FALSE;
    COMPLEX ctemp;
    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 3; n < MAX_N; n++) {
#if defined VERBOSE
              printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);
#endif
		COMPLEX *cx = new COMPLEX[veclen(n, incx)];
		COMPLEX *cy = new COMPLEX[veclen(n, incy)];
		complex < double >*cxd =
		    new complex < double >[veclen(n, incx)];
		complex < double >*cyd =
		    new complex < double >[veclen(n, incy)];
		COMPLEX c, s;
		complex < double > cd, sd;
		set_random_number(&c, &cd);
		set_random_number(&s, &sd);
		int j = 0;
		while (j < MAX_ITER) {
		    set_random_vector(cx, cxd, veclen(n, incx));
		    set_random_vector(cy, cyd, veclen(n, incy));
		    Clacrt(n, cx, incx, cy, incy, c, s);
		    zlacrt_f77(&n, cxd, &incx, cyd, &incy, &cd, &sd);
		    double diff = infnorm(cx, cxd, veclen(n, incx), 1);
		    if (fabs(diff) > EPSILON) {
			printf("error %.16e!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]cx;
		delete[]cy;
		delete[]cxd;
		delete[]cyd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Clacrt test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clacrt_test();
    printf("Clacrt test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
