/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cladiv.debug.cpp,v 1.1 2009/12/21 04:16:32 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

void
Cladiv_test()
{
    int errorflag = FALSE;
    COMPLEX x, y, ret;
    complex<double> xd, yd, retd;

    int count = 100;
    while (count--) {
	set_random_number(&x, &xd);
	set_random_number(&y, &yd);

	ret = Cladiv(x, y);
	retd = zladiv_f77(&xd, &yd);

#if defined VERBOSE_TEST
      cout << "x      " ; printnum (x) ; cout << endl;
      cout << "y      " ; printnum (y) ; cout << endl;
      cout << "ret    " ; printnum (ret) ; cout << endl;

      cout << "xd     " ; printnum (xd) ; cout << endl;
      cout << "yd     " ; printnum (yd) ; cout << endl;
      cout << "retd   " ; printnum (retd) ; cout << endl;
#endif
	if (num_diff(ret, retd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
    }
    if (errorflag == TRUE) {
	printf("Cladiv test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cladiv_test();
    printf("Cladiv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

