/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Claesy.debug.cpp,v 1.1 2009/12/26 04:12:52 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define ITERATION 100

void
Claesy_test()
{
    int errorflag = FALSE;
    REAL rtemp;

    COMPLEX cs1, rt1, rt2;
    COMPLEX a, b, c, sn1, evscal;
    complex <double> cs1d, rt1d, rt2d;
    complex <double> ad, bd, cd, sn1d, evscald;
    int count = 100;
    while (count--) {
#if defined VERBOSE_TEST
    printf("Claesy: general random case\n");
#endif
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	set_random_number(&c, &cd);
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
/* checking adf = ab case (|a-c| = 2|b|) */
#if defined VERBOSE_TEST
    printf("Claesy: |a-c| = 2|b| case\n");
#endif
	set_random_number(&a, &ad);
	set_random_number(&c, &cd);
	b = (a - c) / 2.0;
	bd = (ad - cd) / 2.0;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
/* checking sm = 0 case */
#if defined VERBOSE_TEST
    printf("Claesy: sm = 0 case\n");
#endif
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	c = -a;
	cd = -ad;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
/*zero eigenvalue case */
#if defined VERBOSE_TEST
    printf("Claesy: zero eigenvalue case\n");
#endif
	set_random_number(&a, &ad);
	b = 0.0, c = 0.0;
	bd = cd = 0.0;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error4\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error4\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error4\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error4\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error4\n");
	}
/*zero matrix case */
#if defined VERBOSE_TEST
    printf("Claesy: zero matrix case\n");
#endif
        a = 0.0, b = 0.0, c = 0.0;
	ad = bd = cd = 0.0;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error5\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error5\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error5\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error5\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error5\n");
	}
/*Identity matrix case */
#if defined VERBOSE_TEST
    printf("Claesy: identity matrix case\n");
#endif
	set_random_number(&a, &ad);
	set_random_number(&c, &cd);
	b = 0.0;
	bd = 0.0;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error6\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error6\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error6\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error6\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error6\n");
	}
	if (errorflag == TRUE) {
	    printf("Claesy test failed...\n");
	    exit(-1);
	}
    }
/*Identity matrix case */
#if defined VERBOSE_TEST
    printf("Claesy: diagonal matrix case\n");
#endif
        a = 1.0, c = 1.0;
	b = 0.0;
	ad = cd = 1.0;
	bd = 0.0;
	Claesy(a, b, c, &rt1, &rt2, &evscal, &cs1, &sn1);
	zlaesy_f77(&ad, &bd, &cd, &rt1d, &rt2d, &evscald, &cs1d, &sn1d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error7\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error7\n");
	}
	if (num_diff(cs1, cs1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error7\n");
	}
	if (num_diff(sn1, sn1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error7\n");
	}
	if (num_diff(evscal, evscald) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error7\n");
	}
	if (errorflag == TRUE) {
	    printf("Claesy test failed...\n");
	    exit(-1);
	}
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Claesy_test();
    printf("Claesy test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

