/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clarft.debug.cpp,v 1.1 2009/12/22 11:05:55 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_K 1
#define MAX_K 15
#define MIN_N 0
#define MAX_N 15
#define MAX_LDT 15
#define MAX_LDC 15
#define MAX_LDV 15
#define MAX_ITER 3

void
Clarft_test2(const char *direct, const char *storev)
{
    int errorflag = FALSE;
    int n, k, ldvmin, v_column, ldt, iter, ldv;

    for (k = MIN_K; k <= MAX_K; k++) {
	for (n = k; n <= MAX_N; n++) {
	    if (Mlsame(storev, "C"))
		ldvmin = max(1, n);
	    if (Mlsame(storev, "R"))
		ldvmin = k;
	    for (ldv = ldvmin; ldv <= MAX_LDV; ldv++) {
		for (ldt = k; ldt <= MAX_LDT; ldt++) {
		    if (Mlsame(storev, "C"))
			v_column = k;
		    if (Mlsame(storev, "R"))
			v_column = n;
#if defined VERBOSE_TEST
		    printf ("#direct %s: storev %s, n %d, k %d, ldv %d, ldt %d\n",
			direct, storev, n, k, ldv, ldt);

#endif
		    COMPLEX *T = new COMPLEX[matlen(ldt, k)];
		    COMPLEX *V = new COMPLEX[matlen(ldv, v_column)];
		    COMPLEX *tau = new COMPLEX[veclen(k, 1)];

		    complex<double> *Td = new complex<double>[matlen(ldt, k)];
		    complex<double> *Vd = new complex<double>[matlen(ldv, v_column)];
		    complex<double> *taud = new complex<double>[veclen(k, 1)];


		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(T, Td, matlen(ldt, k));
			set_random_vector(V, Vd, matlen(ldv, v_column));
			set_random_vector(tau, taud, veclen(k, 1));

			zlarft_f77(direct, storev, &n, &k, Vd, &ldv, taud, Td,
			    &ldt);
			Clarft(direct, storev, n, k, V, ldv, tau, T, ldt);
			double diff;

			diff = infnorm(T, Td, matlen(ldt, k), 1);
			if (fabs(diff) > EPSILON*1000.0) {
			    printf("#error in T %.16e!!\n", diff);
			    errorflag = TRUE;
			}
			diff = infnorm(V, Vd, matlen(ldv, v_column), 1);
			if (fabs(diff) > EPSILON*100.0) {
			    printf("#error in V%.16e!!\n", diff);
			    errorflag = TRUE;
			}
		    }
		    delete[]taud;
		    delete[]Vd;
		    delete[]Td;
		    delete[]tau;
		    delete[]V;
		    delete[]T;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Clarft test failed...\n");
	exit(-1);
    }
}

void
Clarft_test()
{
    Clarft_test2("B", "R");
    Clarft_test2("B", "C");
    Clarft_test2("F", "C");
    Clarft_test2("F", "R");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clarft_test();
    printf("Clarft test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
