/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clasr.debug.cpp,v 1.1 2009/12/21 06:09:54 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N     0
#define MAX_N     15
#define MIN_M     0
#define MAX_M     15
#define MAX_LDA   18
#define MAX_ITER  3

void Clasr_test2(const char *side, const char *pivot, const char *direct);

void
Clasr_test2(const char *side, const char *pivot, const char *direct)
{
    int errorflag = FALSE;
    int j = 0;
    int cdim;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int m = MIN_M; m < MAX_M; m++) {
	    for (int lda = max(m, 1); lda < MAX_LDA; lda++) {
#if defined VERBOSE_TEST
		printf
		    ("# n:%d m:%d lda:%d, side %s, pivot %s direct %s\n",
		    n, m, lda, side, pivot, direct);
#endif
		if (Mlsame(side, "L"))
		    cdim = m - 1;
		if (Mlsame(side, "R"))
		    cdim = n - 1;

		COMPLEX *A = new COMPLEX[matlen(lda, n)];
		REAL *c = new REAL[max(cdim, 1)];
		REAL *s = new REAL[max(cdim, 1)];
		complex<double> *Ad = new complex<double>[matlen(lda, n)];
		double *cd = new double[max(cdim, 1)];
		double *sd = new double[max(cdim, 1)];

		j = 0;
		while (j < MAX_ITER) {
		    set_random_vector(A, Ad, matlen(lda, n));
		    set_random_vector(c, cd, max(cdim, 1));
		    set_random_vector(s, sd, max(cdim, 1));
		    Clasr(side, pivot, direct, m, n, c, s, A, lda);
		    zlasr_f77(side, pivot, direct, &m, &n, cd, sd, Ad, &lda);
		    double diff = infnorm(A, Ad, matlen(lda, n), 1);

		    if (fabs(diff) > EPSILON) {
			printf
			    ("# n:%d m:%d lda:%d, side %s, pivot %s direct %s\n",
			    n, m, lda, side, pivot, direct);
			printf("error %e!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]sd;
		delete[]cd;
		delete[]Ad;
		delete[]s;
		delete[]c;
		delete[]A;
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Clasr test failed...\n");
	exit(1);
    }

}
void
Clasr_test(void)
{
    Clasr_test2("L", "V", "F");
    Clasr_test2("R", "V", "F");
    Clasr_test2("L", "T", "F");
    Clasr_test2("R", "T", "F");
    Clasr_test2("L", "B", "F");
    Clasr_test2("R", "B", "F");
    Clasr_test2("L", "V", "B");
    Clasr_test2("R", "V", "B");
    Clasr_test2("L", "T", "B");
    Clasr_test2("R", "T", "B");
    Clasr_test2("L", "B", "B");
    Clasr_test2("R", "B", "B");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clasr_test();
    printf("Clasr test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

