/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Ctrtrs.debug.cpp,v 1.1 2009/12/25 08:30:03 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N 1
#define MAX_N 10		//shold not be so large
#define MIN_LDA 1
#define MAX_LDA 10		//shold not be so large
#define MIN_LDB 1
#define MAX_LDB 10		//shold not be so large
#define MAX_NRHS 10		//shold not be so large
#define MAX_ITER 10

void
Ctrtrs_test2(const char *uplo, const char *trans, const char *diag)
{
    int errorflag = FALSE;
    int iter;
    int n, lda, ldb, nrhs, infod;
    mpackint info;

    for (n = MIN_N; n <= MAX_N; n++) {
	for (lda = max(1, n); lda <= MAX_LDA; lda++) {
	    for (ldb = max(1, n); ldb <= MAX_LDB; ldb++) {
		for (nrhs = 1; nrhs <= MAX_NRHS; nrhs++) {
#if defined VERBOSE_TEST
		    printf
			("# uplo %s, trans %s, diag %s, n %d, lda %d, ldb %d, nrhs %d\n",
			uplo, trans, diag, n, lda, ldb, nrhs);
#endif
		    COMPLEX *A = new COMPLEX[matlen(lda, n)];
		    COMPLEX *B = new COMPLEX[matlen(ldb, nrhs)];

		    complex<double> *Ad = new complex<double>[matlen(lda, n)];
		    complex<double> *Bd = new complex<double>[matlen(ldb, nrhs)];

		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(A, Ad, matlen(lda, n));
			set_random_vector(B, Bd, matlen(ldb, nrhs));

			ztrtrs_f77(uplo, trans, diag, &n, &nrhs, Ad, &lda, Bd,
			    &ldb, &infod);
			Ctrtrs(uplo, trans, diag, n, nrhs, A, lda, B, ldb,
			    &info);

			if (info != infod) {
			    printf("info differ! %d, %d\n", (int)info, infod);
			    printf ("# uplo %s, trans %s, diag %s, n %d, lda %d, ldb %d, nrhs %d\n", uplo, trans, diag, n, lda, ldb, nrhs);
			    errorflag = TRUE;
			    exit(-1);
			}
			double diff;

			diff = infnorm(A, Ad, matlen(lda, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("error in 'A' %.16e!!\n", diff);
			    errorflag = TRUE;
			    exit(-1);
			}
		    }
		    delete[]Ad;
		    delete[]A;
		    delete[]Bd;
		    delete[]B;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Ctrtrs test failed...\n");
	exit(1);
    }
}
void
Ctrtrs_test()
{
    Ctrtrs_test2("U", "N", "N");
    Ctrtrs_test2("U", "N", "U");
    Ctrtrs_test2("U", "T", "N");
    Ctrtrs_test2("U", "T", "U");
    Ctrtrs_test2("U", "C", "N");
    Ctrtrs_test2("U", "C", "U");
    Ctrtrs_test2("L", "N", "N");
    Ctrtrs_test2("L", "N", "U");
    Ctrtrs_test2("L", "T", "N");
    Ctrtrs_test2("L", "T", "U");
    Ctrtrs_test2("L", "C", "N");
    Ctrtrs_test2("L", "C", "U");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Ctrtrs_test();
    printf("Ctrtrs test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
