/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cungql.debug.cpp,v 1.1 2009/12/24 07:40:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N 0
#define MAX_N 15
#define MIN_M 0
#define MAX_M 15
#define MIN_K 0
#define MAX_K 15
#define MIN_LDA 0
#define MAX_LDA 15
#define MAX_ITER 3

void
Cungql_test()
{
    int errorflag = FALSE;
    int iter;
    int m, n, k;
    int lda, infod;
    int lwork, worksized, worksize;
    mpackint info;

    for (m = MIN_M; m <= MAX_M; m++) {
	for (n = MIN_N; n <= m; n++) {
	    for (k = MIN_K; k <= n; k++) {
		for (lda = max(1, m); lda <= MAX_LDA; lda++) {
#if defined VERBOSE_TEST
		    printf("# m %d n %d k %d lda %d\n", m, n, k, lda);
#endif
		    COMPLEX *A = new COMPLEX[matlen(lda, n)];
		    COMPLEX *tau = new COMPLEX[veclen(k, 1)];
		    COMPLEX *work = new COMPLEX[veclen(n, 1) * 1024];

		    complex<double> *Ad = new complex<double>[matlen(lda, n)];
		    complex<double> *taud = new complex<double>[veclen(k, 1)];
		    complex<double> *workd = new complex<double>[veclen(n, 1) * 1024];

		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(A, Ad, matlen(lda, n));
			set_random_vector(tau, taud, veclen(k, 1));
			set_random_vector(work, workd, veclen(n, 1) * 1024);
// these workspace query might not be the same value.
			lwork = -1;
			zungql_f77(&m, &n, &k, Ad, &lda, taud, workd, &lwork,
			    &infod);
			Cungql(m, n, k, A, lda, tau, work, lwork, &info);
			worksize = (int)(cast2double(work[0].re));
			worksized = (int)workd[0].real();
#if defined VERBOSE_TEST
                       printf ("optimized worksize by zungql %d : by Cungql %d.\n", worksized, worksize);
#endif
#ifdef DUMMY
//comparison of workspace is nonsense...
                        if(worksize!=worksized) printf("error in worksize\n");
#endif
			lwork = worksize;
			zungql_f77(&m, &n, &k, Ad, &lda, taud, workd, &lwork,
			    &infod);
			Cungql(m, n, k, A, lda, tau, work, lwork, &info);
			double diff;

			diff = infnorm(A, Ad, matlen(lda, n), 1);
			if (fabs(diff) > EPSILON*100.0) {
			    printf("error in 'A' %.16e!!\n", diff);
			    errorflag = TRUE;
			    exit(1);
			}
			diff = infnorm(tau, taud, veclen(k, 1), 1);
			if (fabs(diff) > EPSILON) {
			    printf("error in 'W' %.16e!!\n", diff);
			    errorflag = TRUE;
			    exit(1);
			}
#ifdef DUMMY
//comparison of workspace is nonsense...
			diff = infnorm(work, workd, veclen(n, 1), 1);
			if (fabs(diff) > EPSILON) {
			    printf("error in 'work' %.16e!!\n", diff);
			    errorflag = TRUE;
			    exit(1);
			}
#endif
		    }
		    delete[]taud;
		    delete[]workd;
		    delete[]Ad;
		    delete[]tau;
		    delete[]work;
		    delete[]A;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cungql test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cungql_test();
    printf("Cungql test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
