/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rlanst.debug.cpp,v 1.3 2009/12/21 02:26:36 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N     0
#define MAX_N     100
#define MAX_ITER  10

void Rlanst_test2(const char *norm);

void
Rlanst_test2(const char *norm)
{
    int errorflag = FALSE;
    int j = 0, ni ;
    REAL Rlanst_ret;
    double dlanst_ret;

    for (mpackint n = MIN_N; n < MAX_N; n++) {
        ni = n; 
#if defined VERBOSE_TEST
	printf("n:%d norm %s\n", (int)n, norm);
#endif
	REAL *d = new REAL[veclen(n, 1)];
	REAL *e = new REAL[veclen(n - 1, 1)];
	double *dd = new double[veclen(n, 1)];
	double *ed = new double[veclen(n - 1, 1)];
	j = 0;
	while (j < MAX_ITER) {
	    set_random_vector(d, dd, veclen(n, 1));
	    set_random_vector(e, ed, veclen(n - 1, 1));
	    Rlanst_ret = Rlanst(norm, n, d, e);
	    dlanst_ret = dlanst_f77(norm, &ni, dd, ed);
	    double diff = num_diff(Rlanst_ret, dlanst_ret);
	    if (fabs(diff) > EPSILON) {
		printf("error %.16e!!\n", diff);
		errorflag = TRUE;
	    }
	    j++;
	}
	delete[]d;
	delete[]dd;
	delete[]e;
	delete[]ed;
    }
    if (errorflag == TRUE) {
	printf("Rlanst test failed...\n");
	exit(1);
    }
}

void
Rlanst_test(void)
{
    Rlanst_test2("M");
    Rlanst_test2("m");
    Rlanst_test2("1");
    Rlanst_test2("O");
    Rlanst_test2("o");
    Rlanst_test2("I");
    Rlanst_test2("i");
    Rlanst_test2("F");
    Rlanst_test2("f");
    Rlanst_test2("E");
    Rlanst_test2("e");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rlanst_test();
    printf("Rlanst test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
