/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rlapy3.debug.cpp,v 1.1 2009/12/21 04:16:32 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

void
Rlapy3_test()
{
    int errorflag = FALSE;
    REAL rtemp;

    REAL Rlapy3_ret, x, y, z;
    double dlapy3_ret, xd, yd, zd;

    int count = 100;

    while (count--) {
	set_random_number(&x, &xd);
	set_random_number(&y, &yd);
	set_random_number(&z, &zd);

	Rlapy3_ret = Rlapy3(x, y, z);
	dlapy3_ret = dlapy3_f77(&xd, &yd, &zd);

#if defined VERBOSE_TEST
      cout << "x      " ; printnum (x) ; cout << endl;
      cout << "y      " ; printnum (y) ; cout << endl;
      cout << "z      " ; printnum (z) ; cout << endl;
      cout << "Rlapy3 " ; printnum (Rlapy3_ret) ; cout << endl;
      cout << "xd     " ; printnum (xd) ; cout << endl;
      cout << "yd     " ; printnum (yd) ; cout << endl;
      cout << "zd     " ; printnum (zd) ; cout << endl;
      cout << "dlapy3 "  ; printnum (dlapy3_ret) ; cout << endl;
#endif
	if (num_diff(Rlapy3_ret, dlapy3_ret) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
    }
    if (errorflag == TRUE) {
	printf("Rlapy3 test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rlapy3_test();
    printf("Rlapy3 test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

