/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rlassq.debug.cpp,v 1.3 2009/12/18 07:09:45 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_INCX  1
#define MAX_INCX  10
#define MIN_N     1
#define MAX_N     100
#define MAX_ITER 100

void
Rlassq_test()
{
    int errorflag = FALSE;
    REAL scale, sumsq;
    double scaled, sumsqd;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n <= MAX_N; n++) {
	    REAL *x = new REAL[veclen(n, incx)];
	    double *xd = new double[veclen(n, incx)];
#if defined VERBOSE_TEST
	    printf("# incx: %d, n: %d\n", incx, n);
#endif
	    int j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(x, xd, veclen(n, incx));
		set_random_number(&scale, &scaled);
		set_random_number(&sumsq, &sumsqd);
		Rlassq(n, x, incx, &scale, &sumsq);
		dlassq_f77(&n, xd, &incx, &scaled, &sumsqd);
		double diff1 = rel_diff(scale, scaled);
		if (fabs(diff1) > EPSILON) {
		    printf("error1 scale %.16e!!\n", diff1);
		    errorflag = TRUE;
		}
		diff1 = rel_diff(sumsq, sumsqd);
		if (fabs(diff1) > EPSILON) {
		    printf("error1 in sumsq %.16e %.16e!!\n", diff1, sumsqd);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]x;
	    delete[]xd;
	}
    }
    if (errorflag == TRUE) {
	printf("Rlassq test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rlassq_test();
    printf("Rlassq test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
