/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rsterf.debug.cpp,v 1.3 2009/09/23 01:47:10 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N      0
#define MAX_N     100
#define MAX_ITER  100

void
Rsterf_test(void)
{
    int errorflag = FALSE;
    int j = 0;
    int infod;
    double diff;
    mpackint info;

    for (int n = MIN_N; n < MAX_N; n++) {
	REAL *D = new REAL[veclen(n, 1)];
	REAL *E = new REAL[veclen(n - 1, 1)];
	double *Dd = new double[veclen(n, 1)];
	double *Ed = new double[veclen(n - 1, 1)];
#if defined VERBOSE_TEST
	printf("# n:%d\n", n);
#endif
	j = 0;
	while (j < MAX_ITER) {
	    set_random_vector(D, Dd, veclen(n, 1));
	    set_random_vector(E, Ed, veclen(n - 1, 1));
	    Rsterf(n, D, E, &info);
	    dsterf_f77(&n, Dd, Ed, &infod);
	    if (info < 0) {
		printf("info %d error\n", -(int)info);
		errorflag = TRUE;
	    }
	    if (info != infod) {
		printf("info differ! %d, %d\n", (int)info, infod);
		errorflag = TRUE;
	    }
	    diff = infnorm(D, Dd, veclen(n, 1), 1);
	    if (fabs(diff) > EPSILON) {
		printf("eigenvec error %e!!\n", diff);
		errorflag = TRUE;
	    }
	    j++;
	}
	delete[]D;
	delete[]E;
	delete[]Dd;
	delete[]Ed;
    }
    if (errorflag == TRUE) {
	printf("Rsterf test failed...\n");
	exit(1);
    }
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rsterf_test();
    printf("Rsterf test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
